
import enum
from pydantic import BaseModel, validator
from typing import List, Optional, Tuple, Any



class GlobalOption(BaseModel):
    title: str
    author: Optional[str]
    date: Optional[str]
    input_file_dir: Optional[str]
    input_file_name: Optional[str]
    matplot: bool = True
    kernal: str = "python3"
    log_level: str = "debug"
    fig_dir: str = "figures"
    css: str = "bootstrap"
    output_format: Optional[str]
    output_file_name: Optional[str]
    output_file_dir: Optional[str]
    
    @validator('log_level')
    def fix_option_for_log(cls, v:str):
        if v.lower() not in ('notset', "debug", 'info', 'warning', 'error', 'critical'):
            raise ValueError('must contain a space')
        return v.title()



class ChunkOption(BaseModel):
    name: Optional[str]
    echo: bool = True
    render: bool = True
    run: bool = True
    fig: bool =  False
    fig_size: Tuple =  (6, 4)
    fig_dpi: int = 200
    fig_caption: Optional[str] 
    fig_position: Optional[str] = "htpb"
    fig_env: Optional[str]
    fig_width: Optional[str]
    con_string: Optional[str] = "zen_con"

BOOLENCHUNCKOPTIONS = ['run', 'echo','render', 'fig']


class Chunk(BaseModel):
    type: str
    string_: str
    options: Optional[ChunkOption]


class ParseData(BaseModel):
    global_options: Optional[GlobalOption]
    chunks: List[Chunk]

class ReadData(BaseModel):
    global_options: Optional[GlobalOption]
    data: str


class ExecuatedChucnkOut(BaseModel):
    output_type: str
    data: dict

class ExecuatedChunk(BaseModel):
    chunk: Chunk
    results: List[ExecuatedChucnkOut]

class ExecutedData(BaseModel):
    global_options: Optional[GlobalOption]
    chunks: List[ExecuatedChunk]


class OrganizedChunk(BaseModel):
    type: str
    str_data: str = ""
    complex_data: dict = {}


class OrganizedData(BaseModel):
    global_options: Optional[GlobalOption]
    chunks: List[OrganizedChunk]

class OrganizedChuckType(enum.Enum):
    Markdown = "markdown"
    Code = "code"
    Result = "se_data"
    Table = "e_data"
    HTML = "html_data"
    Plot = "plot"
            


import os
from zen_knit.data_types import ReadData, GlobalOption


class BaseReader:
    def __init__(self, input_file, output_file_dir = None) -> None:        
        full_path = self._build_input_file(input_file)
        input_file_path , input_file_name = self._build_input_file_folder(full_path)
        self.raw_data = ReadData(
            global_options=GlobalOption(
                title = "test",
                input_file_dir=input_file_path,
                input_file_name=input_file_name,
                output_file_dir = output_file_dir,
            ),
            data=self._read(full_path)
        )
        
    def _build_input_file(self, input_file):
        return os.path.abspath(input_file)

    def _build_input_file_folder(self, full_path):
        return os.path.split(full_path)

    def _read(self, full_path):
        with open(full_path, "r") as f:
            contents = f.read()
        return contents
        



import io
from zen_knit.data_types import OrganizedData, GlobalOption, OrganizedChunk



class BaseFormatter:
    def __init__(self, organized_data: OrganizedData):
        self.organized_data = organized_data
        self.formatted_doc = ""
        self.header = ""
        self.footer = ""
        self.subheader = ""


    def run(self):
        self._parsetitle(self.organized_data.global_options)
        self._format_doc()
        self._build_doc()
        file_ = self.write_file()
        self._post_process(file_)

    
    def _parsetitle(self, global_option:GlobalOption):
        pass

    def _format_docchunk(self, content:OrganizedChunk):
        pass
    
    def _format_codechunks(self, content:OrganizedChunk):
        pass
    
    def _format_image(self, content:OrganizedChunk):
        pass

    def _format_html(self, content:OrganizedChunk):
        pass

    def _build_doc(self):
        pass 
    
    def _format_doc(self):
        chunk: OrganizedChunk
        for chunk in self.organized_data.chunks:
            if chunk.type == "markdown":
                t = self._format_docchunk(chunk)
                self.formatted_doc = self.formatted_doc + "\n" + t
            elif chunk.type == "code":
                t = self._format_codechunks(chunk)
                self.formatted_doc = self.formatted_doc + "\n" + t
            elif chunk.type == "se_data":
                t = self._format_codechunks(chunk)
                self.formatted_doc = self.formatted_doc + "\n" + t
            elif chunk.type == "e_data":
                t = self._format_docchunk(chunk)
                self.formatted_doc = self.formatted_doc + "\n" + t
            elif chunk.type == "plot":
                t = self._format_image(chunk)
                self.formatted_doc = self.formatted_doc + "\n" + t
            elif chunk.type == "html_data":
                t = self._format_html(chunk)
                self.formatted_doc = self.formatted_doc + "\n" + t
            else:
                print("Not right format")

    def write_file(self):
        fd = self.organized_data.global_options.output_file_dir
        fn = self.organized_data.global_options.output_file_name
        file_  = f"{fd}/{fn}"
        with io.open(file_, 'wt', encoding='utf-8') as f:
            f.write(self.formatted_doc)
        
    def _post_process(self, file_):
        pass
    #     markdown_file = self.executed_data.global_options.input_file_name.split(".")[0] + ".md"
    #     markdown_file = os.path.join(self.executed_data.global_options.output_file_dir , markdown_file)
    #     with open(markdown_file, "w") as f:
    #         text = "\n".join(self.formatted_doc)
    #         f.write(text)
                


from zen_knit.formattor.base_formatter import BaseFormatter
from zen_knit.formattor.html_formatter import HTMLFormatter
from zen_knit.formattor.markdown_formtter import MarkDownFormatter
from zen_knit.formattor.tex_formatter import TexFormatter

mime_extensions = {"image/png" : "png",
                   "image/jpg" : "jpg"}


my_formatter = {"pdf": TexFormatter,
            "html": HTMLFormatter,
            "markdown":MarkDownFormatter}


def get_formatter(output) -> BaseFormatter:
    return my_formatter[output]


from io import RawIOBase
from posixpath import split
import re
import datetime
import ast
from zen_knit.data_types import GlobalOption, ReadData, ChunkOption, Chunk, ParseData, BOOLENCHUNCKOPTIONS

def option_parser(raw: str) -> ChunkOption:
    options = raw.split("{")[1].split("}")[0].replace(" ", "").split(",")
    option_dict = {}
    for option in options:
        if "=" not in option :
            continue
        passed_option, value = option.split("=")
        if passed_option in BOOLENCHUNCKOPTIONS:
            value = (value == "true") or (value == "True") or (value == "t") or (value == "T")
        option_dict[passed_option] = value
    selected_options = ChunkOption(**option_dict)
    return selected_options


class BaseParser:
    
    def __init__(self, data:ReadData) -> None:
        self.raw_data = data
        self.parsed_data = None
        self.starter_line = "^```\{.*?\}\s*$"
        self.ender_line = "^```$"
        self.title_info = {}
        self._parse_data()

    def _parse_title(self, title_info):
        title_present = False
        for t in title_info.splitlines():
            split_t = t.split(":")
            
            if len(split_t) == 2:
                split_t[1] = split_t[1].lstrip()
                if "title" == split_t[0]:
                    self.raw_data.global_options.__setattr__("title", split_t[1])
                    title_present = True
                if "author" == split_t[0]:
                    self.raw_data.global_options.__setattr__("author", split_t[1])
                if "date" == split_t[0]:
                    date1 = split_t[1]
                    if  "CURRENT_DATE" in date1:
                        date1 =  datetime.datetime.now().strftime("%d %B, %Y")
                    elif "datetime" in date1:
                        date1 = eval(date1)
                    self.raw_data.global_options.__setattr__("date", date1)
                if "output" == split_t[0]:
                    self.raw_data.global_options.__setattr__("output_format", split_t[1])
            else:
                if split_t[0] in ('title', 'author', 'date', 'output'):
                    raise Exception(f"Not right format for {split[1]}")
                
            if 'css' in split_t[0]:
                css = ":".join(split_t[1:])
                self.raw_data.global_options.__setattr__("css", css)

        
        if (not title_present) and (self.raw_data.global_options.title == "test"):
            raise Exception("Title is not provided")

    def _parse_data(self):
        started = False
        current_strings = []
        temp_string = ''
        type_ = None
        option = option_parser("{}")
        if re.match("^---$", self.raw_data.data[:3]):
            _, title, raw_data = re.split("---\n", self.raw_data.data)
            self._parse_title(title)
        else:
            raw_data = self.raw_data.data
        for _, raw in enumerate(raw_data.splitlines()):
            if re.match(self.starter_line, raw):
                if started:
                    raise Exception("code is already started")
                else:
                    current_strings.append(Chunk(type= 'text' if type_ is None else type_,
                                                 string_= temp_string, 
                                                 options= option))
                    temp_string = ''
                    started = True
                    if ('{sql' in raw.lower()):
                        type_ = 'sql'
                    else:
                        type_ = 'code'  
                    option = option_parser(raw)

            else:
                if re.match(self.ender_line, raw):
                    if started:
                        current_strings.append(Chunk(type= 'text' if type_ is None else type_,
                                                 string_= temp_string, 
                                                 options= option))
                        temp_string = ''
                        started = False
                        type_ = None
                        option = option_parser("{}")
                        

                    else:
                        raise Exception("code is not started")
                else:
                    if temp_string != "":
                        temp_string = temp_string + "\n" + raw
                    else:
                        temp_string = raw
        
        current_strings.append(Chunk(type= 'text' if type_ is None else type_,
                                                 string_= temp_string, 
                                                 options= option))

        self.parsed_data = ParseData(global_options=self.raw_data.global_options, chunks=current_strings)



import os
import click

from zen_knit.executor import BaseExecutor
from zen_knit.formattor import get_formatter
from zen_knit.formattor.base_formatter import BaseFormatter
from zen_knit.organizer import BaseOrganizer
from zen_knit.parser import BaseParser
from zen_knit.reader import BaseReader

@click.command()
@click.option("-f", '--file', help='Pyz file name with path or file name for current dirctory', required=True)
@click.option("-ofd", '--output_file_dir', help='where output you want to put it, if not provided, current working directory will be used')
def knit(file, output_file_dir):
    if output_file_dir is None:
        output_file_dir = os.getcwd()

    output_file_dir = os.path.abspath(output_file_dir)
    file = os.path.abspath(file)

    br = BaseReader(file, output_file_dir)
    bp = BaseParser(br.raw_data)
    be = BaseExecutor(bp.parsed_data)
    bo = BaseOrganizer(be.excuted_data)
    f = get_formatter(bo.organized_data.global_options.output_format)
    bf:BaseFormatter # or its derivative class
    bf = f(bo.organized_data)
    bf.run()


import io
from nbconvert import filters
from zen_knit.data_types import ChunkOption, ExecutedData, GlobalOption, OrganizedChunk
from zen_knit.formattor.base_formatter import BaseFormatter


class MarkDownFormatter(BaseFormatter):
    def __init__(self, executed_data: ExecutedData):
        self.formatted_doc = []
        self.header = ""
    
    def _parsetitle(self, global_option:GlobalOption):
        title = global_option.title
        author = global_option.author
        date = global_option.date
        self.header += f"% {title}"
        if author is not None:
            self.header += f"% {author}"
        if date is not None:
            self.header += f"% {date}"

    def _format_docchunk(self, content:OrganizedChunk):
        return content.str_data
    
        
    def _format_codechunks(self, content:OrganizedChunk):
        t = content.str_data
        if content.type == "code":
            return f"```python \n {t} \n ```"
        else:
            return f"``` \n {t} \n ```"

    def _format_image(self, content:OrganizedChunk):
        result = ""
        figstring = ""
        options: ChunkOption
        options = content.complex_data["options"]
        for fig in content.complex_data["plot"]:
            if options.fig_caption:
                result += f"[{options.fig_caption}][{fig}]"
            else:
                result += f"[][{fig}]"

        return result
    

    def _build_doc(self):
        self.formatted_doc =  self.header + self.formatted_doc
        
    def write_file(self):
        fd = self.organized_data.global_options.output_file_dir
        fn = self.organized_data.global_options.output_file_name
        file_  = f"{fd}/{fn}"
        with io.open(file_, 'wt', encoding='utf-8') as f:
            f.write(self.formatted_doc)
        

    #     markdown_file = self.executed_data.global_options.input_file_name.split(".")[0] + ".md"
    #     markdown_file = os.path.join(self.executed_data.global_options.output_file_dir , markdown_file)
    #     with open(markdown_file, "w") as f:
    #         text = "\n".join(self.formatted_doc)
    #         f.write(text)
                


import markdown


class MathPattern(markdown.inlinepatterns.Pattern):
    def __init__(self):
        markdown.inlinepatterns.Pattern.__init__(self, r'(?<!\\)(\$\$?)(.+?)\2')

    def handleMatch(self, m):
        node = markdown.util.etree.Element('span class="math"')
        # node.text = markdown.util.AtomicString(m.group(2) + m.group(3) + m.group(2))
        if m.group(2) == "$$":
            node.text = markdown.util.AtomicString("\[" + m.group(3) + "\]")
        else:
            node.text = markdown.util.AtomicString("\(" + m.group(3) + "\)")
        return node


class MathExtension(markdown.Extension):

    def extendMarkdown(self, md, md_globals):
        md.inlinePatterns.add('math', MathPattern(), '<escape')


# -*- coding: utf-8 -*-

#Modified from skeleton from getskeleton.com
#Smaller headers
css = u"""
@font-face {
  font-family: 'Raleway';
  font-style: normal;
  font-weight: 300;
  src: local('Raleway Light'), local('Raleway-Light'), url(http://fonts.gstatic.com/s/raleway/v9/-_Ctzj9b56b8RgXW8FArifk_vArhqVIZ0nv9q090hN8.woff2) format('woff2');
}
@font-face {
  font-family: 'Raleway';
  font-style: normal;
  font-weight: 400;
  src: local('Raleway'), url(http://fonts.gstatic.com/s/raleway/v9/0dTEPzkLWceF7z0koJaX1A.woff2) format('woff2');
}
@font-face {
  font-family: 'Raleway';
  font-style: normal;
  font-weight: 600;
  src: local('Raleway SemiBold'), local('Raleway-SemiBold'), url(http://fonts.gstatic.com/s/raleway/v9/xkvoNo9fC8O2RDydKj12b_k_vArhqVIZ0nv9q090hN8.woff2) format('woff2');
}
html {
  font-family: sans-serif; /* 1 */
  -ms-text-size-adjust: 100%; /* 2 */
  -webkit-text-size-adjust: 100%; /* 2 */
}
body {
  margin: 0;
}
article,
aside,
details,
figcaption,
figure,
footer,
header,
hgroup,
main,
menu,
nav,
section,
summary {
  display: block;
}
audio,
canvas,
progress,
video {
  display: inline-block; /* 1 */
  vertical-align: baseline; /* 2 */
}
audio:not([controls]) {
  display: none;
  height: 0;
}
[hidden],
template {
  display: none;
}
a:active,
a:hover {
  outline: 0;
}
abbr[title] {
  border-bottom: 1px dotted;
}
b,
strong {
  font-weight: bold;
}
dfn {
  font-style: italic;
}
h1 {
  font-size: 2em;
  margin: 0.67em 0;
}
mark {
  background: #ff0;
  color: #000;
}
small {
  font-size: 80%;
}
sub,
sup {
  font-size: 75%;
  line-height: 0;
  position: relative;
  vertical-align: baseline;
}
sup {
  top: -0.5em;
}
sub {
  bottom: -0.25em;
}
img {
  border: 0;
}
svg:not(:root) {
  overflow: hidden;
}
figure {
  margin: 1em 40px;
}
hr {
  -moz-box-sizing: content-box;
  box-sizing: content-box;
  height: 0;
}
pre {
  overflow: auto;
}
code,
kbd,
pre,
samp {
  font-family: monospace, monospace;
  font-size: 1em;
}
button,
input,
optgroup,
select,
textarea {
  color: inherit; /* 1 */
  font: inherit; /* 2 */
  margin: 0; /* 3 */
}
button {
  overflow: visible;
}
button,
select {
  text-transform: none;
}
button,
html input[type="button"], /* 1 */
input[type="reset"],
input[type="submit"] {
  -webkit-appearance: button; /* 2 */
  cursor: pointer; /* 3 */
}
button[disabled],
html input[disabled] {
  cursor: default;
}
button::-moz-focus-inner,
input::-moz-focus-inner {
  border: 0;
  padding: 0;
}
input {
  line-height: normal;
}
input[type="checkbox"],
input[type="radio"] {
  box-sizing: border-box; /* 1 */
  padding: 0; /* 2 */
}
input[type="number"]::-webkit-inner-spin-button,
input[type="number"]::-webkit-outer-spin-button {
  height: auto;
}
input[type="search"] {
  -webkit-appearance: textfield; /* 1 */
  -moz-box-sizing: content-box;
  -webkit-box-sizing: content-box; /* 2 */
  box-sizing: content-box;
}
input[type="search"]::-webkit-search-cancel-button,
input[type="search"]::-webkit-search-decoration {
  -webkit-appearance: none;
}
fieldset {
  border: 1px solid #c0c0c0;
  margin: 0 2px;
  padding: 0.35em 0.625em 0.75em;
}
legend {
  border: 0; /* 1 */
  padding: 0; /* 2 */
}
textarea {
  overflow: auto;
}
optgroup {
  font-weight: bold;
}
table {
  border-collapse: collapse;
  border-spacing: 0;
}
td,
th {
  padding: 0;
}


/*
* Skeleton V2.0.4
* Copyright 2014, Dave Gamache
* www.getskeleton.com
* Free to use under the MIT license.
* http://www.opensource.org/licenses/mit-license.php
* 12/29/2014
*/
.container {
  position: relative;
  width: 100%;
  max-width: 960px;
  margin: 0 auto;
  padding: 0 20px;
  box-sizing: border-box; }
.column,
.columns {
  width: 100%;
  float: left;
  box-sizing: border-box; }
@media (min-width: 400px) {
  .container {
    width: 85%;
    padding: 0; }
}
@media (min-width: 550px) {
  .container {
    width: 80%; }
  .column,
  .columns {
    margin-left: 4%; }
  .column:first-child,
  .columns:first-child {
    margin-left: 0; }

  .one.column,
  .one.columns                    { width: 4.66666666667%; }
  .two.columns                    { width: 13.3333333333%; }
  .three.columns                  { width: 22%;            }
  .four.columns                   { width: 30.6666666667%; }
  .five.columns                   { width: 39.3333333333%; }
  .six.columns                    { width: 48%;            }
  .seven.columns                  { width: 56.6666666667%; }
  .eight.columns                  { width: 65.3333333333%; }
  .nine.columns                   { width: 74.0%;          }
  .ten.columns                    { width: 82.6666666667%; }
  .eleven.columns                 { width: 91.3333333333%; }
  .twelve.columns                 { width: 100%; margin-left: 0; }

  .one-third.column               { width: 30.6666666667%; }
  .two-thirds.column              { width: 65.3333333333%; }

  .one-half.column                { width: 48%; }

  /* Offsets */
  .offset-by-one.column,
  .offset-by-one.columns          { margin-left: 8.66666666667%; }
  .offset-by-two.column,
  .offset-by-two.columns          { margin-left: 17.3333333333%; }
  .offset-by-three.column,
  .offset-by-three.columns        { margin-left: 26%;            }
  .offset-by-four.column,
  .offset-by-four.columns         { margin-left: 34.6666666667%; }
  .offset-by-five.column,
  .offset-by-five.columns         { margin-left: 43.3333333333%; }
  .offset-by-six.column,
  .offset-by-six.columns          { margin-left: 52%;            }
  .offset-by-seven.column,
  .offset-by-seven.columns        { margin-left: 60.6666666667%; }
  .offset-by-eight.column,
  .offset-by-eight.columns        { margin-left: 69.3333333333%; }
  .offset-by-nine.column,
  .offset-by-nine.columns         { margin-left: 78.0%;          }
  .offset-by-ten.column,
  .offset-by-ten.columns          { margin-left: 86.6666666667%; }
  .offset-by-eleven.column,
  .offset-by-eleven.columns       { margin-left: 95.3333333333%; }

  .offset-by-one-third.column,
  .offset-by-one-third.columns    { margin-left: 34.6666666667%; }
  .offset-by-two-thirds.column,
  .offset-by-two-thirds.columns   { margin-left: 69.3333333333%; }

  .offset-by-one-half.column,
  .offset-by-one-half.columns     { margin-left: 52%; }

}
html {
  font-size: 62.5%; }
body {
  font-size: 1.5em; /* currently ems cause chrome bug misinterpreting rems on body element */
  line-height: 1.6;
  font-weight: 400;
  font-family: "Raleway", "HelveticaNeue", "Helvetica Neue", Helvetica, Arial, sans-serif;
  color: #222; }
h1, h2, h3, h4, h5, h6 {
  margin-top: 0;
  margin-bottom: 2rem;
  font-weight: 300; }
h1 { font-size: 3.6rem; line-height: 1.2;  letter-spacing: -.1rem;}
h2 { font-size: 3.4rem; line-height: 1.25; letter-spacing: -.1rem; }
h3 { font-size: 3.2rem; line-height: 1.3;  letter-spacing: -.1rem; }
h4 { font-size: 2.8rem; line-height: 1.35; letter-spacing: -.08rem; }
h5 { font-size: 2.4rem; line-height: 1.5;  letter-spacing: -.05rem; }
h6 { font-size: 1.5rem; line-height: 1.6;  letter-spacing: 0; }

p {
  margin-top: 0; }
a {
  color: #1EAEDB; }
a:hover {
  color: #0FA0CE; }
.button,
button,
input[type="submit"],
input[type="reset"],
input[type="button"] {
  display: inline-block;
  height: 38px;
  padding: 0 30px;
  color: #555;
  text-align: center;
  font-size: 11px;
  font-weight: 600;
  line-height: 38px;
  letter-spacing: .1rem;
  text-transform: uppercase;
  text-decoration: none;
  white-space: nowrap;
  background-color: transparent;
  border-radius: 4px;
  border: 1px solid #bbb;
  cursor: pointer;
  box-sizing: border-box; }
.button:hover,
button:hover,
input[type="submit"]:hover,
input[type="reset"]:hover,
input[type="button"]:hover,
.button:focus,
button:focus,
input[type="submit"]:focus,
input[type="reset"]:focus,
input[type="button"]:focus {
  color: #333;
  border-color: #888;
  outline: 0; }
.button.button-primary,
button.button-primary,
input[type="submit"].button-primary,
input[type="reset"].button-primary,
input[type="button"].button-primary {
  color: #FFF;
  background-color: #33C3F0;
  border-color: #33C3F0; }
.button.button-primary:hover,
button.button-primary:hover,
input[type="submit"].button-primary:hover,
input[type="reset"].button-primary:hover,
input[type="button"].button-primary:hover,
.button.button-primary:focus,
button.button-primary:focus,
input[type="submit"].button-primary:focus,
input[type="reset"].button-primary:focus,
input[type="button"].button-primary:focus {
  color: #FFF;
  background-color: #1EAEDB;
  border-color: #1EAEDB; }
input[type="email"],
input[type="number"],
input[type="search"],
input[type="text"],
input[type="tel"],
input[type="url"],
input[type="password"],
textarea,
select {
  height: 38px;
  padding: 6px 10px; /* The 6px vertically centers text on FF, ignored by Webkit */
  background-color: #fff;
  border: 1px solid #D1D1D1;
  border-radius: 4px;
  box-shadow: none;
  box-sizing: border-box; }
/* Removes awkward default styles on some inputs for iOS */
input[type="email"],
input[type="number"],
input[type="search"],
input[type="text"],
input[type="tel"],
input[type="url"],
input[type="password"],
textarea {
  -webkit-appearance: none;
     -moz-appearance: none;
          appearance: none; }
textarea {
  min-height: 65px;
  padding-top: 6px;
  padding-bottom: 6px; }
input[type="email"]:focus,
input[type="number"]:focus,
input[type="search"]:focus,
input[type="text"]:focus,
input[type="tel"]:focus,
input[type="url"]:focus,
input[type="password"]:focus,
textarea:focus,
select:focus {
  border: 1px solid #33C3F0;
  outline: 0; }
label,
legend {
  display: block;
  margin-bottom: .5rem;
  font-weight: 600; }
fieldset {
  padding: 0;
  border-width: 0; }
input[type="checkbox"],
input[type="radio"] {
  display: inline; }
label > .label-body {
  display: inline-block;
  margin-left: .5rem;
  font-weight: normal; }
ul {
  list-style: circle inside; }
ol {
  list-style: decimal inside; }
ol, ul {
  padding-left: 0;
  margin-top: 0; }
ul ul,
ul ol,
ol ol,
ol ul {
  margin: 1.5rem 0 1.5rem 3rem;
  font-size: 90%; }
li {
  margin-bottom: 1rem; }
th,
td {
  padding: 12px 15px;
  text-align: left;
  border-bottom: 1px solid #E1E1E1; }
th:first-child,
td:first-child {
  padding-left: 0; }
th:last-child,
td:last-child {
  padding-right: 0; }
button,
.button {
  margin-bottom: 1rem; }
input,
textarea,
select,
fieldset {
  margin-bottom: 1.5rem; }
pre,
blockquote,
dl,
figure,
table,
p,
ul,
ol,
form {
  margin-bottom: 2.5rem; }
.u-full-width {
  width: 100%;
  box-sizing: border-box; }
.u-max-full-width {
  max-width: 100%;
  box-sizing: border-box; }
.u-pull-right {
  float: right; }
.u-pull-left {
  float: left; }
hr {
  margin-top: 3rem;
  margin-bottom: 3.5rem;
  border-width: 0;
  border-top: 1px solid #E1E1E1; }
.container:after,
.row:after,
.u-cf {
  content: "";
  display: table;
  clear: both; }

pre {
  display: block;
  padding: 9.5px;
  margin: 0 0 10px;
  font-size: 13px;
  line-height: 1.42857143;
  color: #333;
  word-break: break-all;
  word-wrap: break-word;
  background-color: #f5f5f5;
  border: 1px solid #ccc;
  border-radius: 4px;
}
code,
kbd,
pre,
samp {
  font-family: Menlo, Monaco, Consolas, "Courier New", monospace;
}
code {
  padding: 2px 4px;
  font-size: 90%;
  color: #c7254e;
  background-color: #f9f2f4;
  border-radius: 4px;
}

@media (min-width: 400px) {}
@media (min-width: 550px) {}
@media (min-width: 750px) {}
@media (min-width: 1000px) {}
@media (min-width: 1200px) {}
"""



import os
from zen_knit.data_types import Chunk, ChunkOption, ExecuatedChucnkOut, ParseData, ExecuatedChunk, ExecutedData
from ipykernel.inprocess import InProcessKernelManager
from jupyter_client import KernelManager, BlockingKernelClient
from queue import Empty
from nbformat.v4 import output_from_msg
import re


database_query = """
from sqlalchemy import create_engine 
import pandas as pd 
engine = create_engine('{my_connection}')
with engine.begin() as con:
    {var_name} = pd.read_sql('{query}', con )
{var_name}.{result_in} """

class BaseExecutor(object):
    
    def __init__(self, data: ParseData):
        self.new_line_regex = re.compile("^\S*")
        self.data = data
        self.excuted_data = ExecutedData(global_options=data.global_options, chunks=[])
        self._set_up_kernal()
        if self.data.global_options.matplot:
            self._preload_matplot()
        self._run_chucks()
        self._close_kernal()
        
    
    def _set_up_kernal(self):
        global_options = self.data.global_options
        self.km = InProcessKernelManager(kernel_name=global_options.kernal)
        log_level = global_options.log_level
        if log_level != 'notset':
            log_file =  os.path.join(global_options.input_file_dir, "log.txt")
            f = open(log_file, "w")
        else:
            f = open(os.devnull, 'w')
        self.km.start_kernel(cwd=global_options.input_file_dir, stderr=f)

        self.kc = self.km.client()
        self.kc.start_channels()
        try:
            _ = self.kc.wait_for_ready()
        except RuntimeError:
            print("Timeout from starting kernel\nTry restarting python session and running it again")
            self._close_kernal()
            raise

        self.kc.allow_stdin = False

    def _close_kernal(self):
        self.kc.stop_channels()
        self.km.shutdown_kernel()


    def _run_chucks(self):
        for chunk in self.data.chunks:
            self._run_chunck(chunk)

    def _run_and_save(self, code:str,  ec:ExecuatedChunk, type_:str):
        ran_code = self._run_code(code)
        if ran_code is not None:
            for r in ran_code:
                eco = ExecuatedChucnkOut(
                    output_type=type_,
                    data= r
                )
                ec.results.append(eco)

    def _run_sql(self, chunk:Chunk, ec:ExecuatedChunk):
        query = chunk.string_.replace("\n", " ")
        con_string =  chunk.options.con_string
        my_connection = os.getenv(con_string)
        
        var_name = chunk.options.name or "zen"
        if self.data.global_options.output_format == "pdf":
            result_in = "to_latex()"
        else:
            result_in = "to_html()"
        code = database_query.format(my_connection=my_connection, query=query, var_name=var_name, result_in=result_in)
        self._run_and_save(code, ec, "code")

    def _run_code_chunk(self, chunk:Chunk, ec:ExecuatedChunk):
        code = ""
        for partial_string in chunk.string_.splitlines():
            if self.new_line_regex.search(partial_string, 1):
                if code != "":
                    self._run_and_save(code, ec, chunk.type)
                    code = ""
                code = partial_string
            else:
                code = code + "\n" + partial_string
        
        self._run_and_save(code, ec, chunk.type)
    
    def _run_chunck(self, chunk:Chunk):
        self._pre_run(chunk)
        ec = ExecuatedChunk(chunk=chunk, results=[])

        if chunk.options.echo:
            eco = ExecuatedChucnkOut(
                output_type=chunk.type,
                data= {'code_text_raw': chunk.string_}
            )
            ec.results.append(eco)

        if (chunk.options.run) & (chunk.type == "sql"):
            self._run_sql(chunk, ec)

        if (chunk.options.run) & (chunk.type == "code"):
            self._run_code_chunk(chunk, ec)
        
        self.excuted_data.chunks.append(ec)    
        
    def _run_code(self, code):
        msg_id = self._send_to_kernal(code)
        return self._pull_data_from_kernal(msg_id)

    def _preload_matplot(self):
        init_matplotlib = """
            %matplotlib inline
            from IPython.display import set_matplotlib_formats
            set_matplotlib_formats('png', 'pdf', 'svg')
            import matplotlib
            """
        self._run_code(init_matplotlib)

    def _pre_run(self, chunk:Chunk):
        options = chunk.options
        if options.fig:
            fig_size = "matplotlib.rcParams.update({'figure.figsize' : ({height}, {width})})".foramt(height=options.fig_size[0], width=options.fig_size[1])
            f_dpi = "matplotlib.rcParams.update({'figure.dpi' : {dpi}})".format(dpi=options.fig_dpi)
            code = fig_size + "\n" + f_dpi
            self._run_code(code)

    def _send_to_kernal(self, code):
        msg_id = self.kc.execute(code, store_history=False)
        while True:
            try:
                msg = self.kc.get_shell_msg(timeout=10)
            except Empty:
                pass
            if msg['parent_header'].get('msg_id') == msg_id:
                break
            else:
                # not our reply
                continue
        return msg_id
    

    def _pull_data_from_kernal(self, msg_id):
        outs = []
        while True:
            try:
                msg = self.kc.iopub_channel.get_msg(block=True, timeout=4)
            except Empty:
                raise RuntimeError("Timeout waiting for IOPub output")

            if msg['parent_header'].get('msg_id') != msg_id and msg['msg_type'] != "stream":
                continue

            msg_type = msg['msg_type']
            content = msg['content']


            if msg_type == 'status':
                if content['execution_state'] == 'idle':
                    break
                else:
                    continue
            elif msg_type == 'execute_input':
                continue
            elif msg_type == 'clear_output':
                self.excuted_data.chunks = []
                continue
            elif msg_type.startswith('comm'):
                continue

            try:
                out = output_from_msg(msg)
            except ValueError:
                print("unhandled iopub msg: " + msg_type)
            else:
                outs.append(out)
        return outs



import base64
import requests
from zen_knit.formattor import html_support
try:
    import markdown
except ImportError:
    message = "You'll need to install python markdown in order to use markdown to html formatter\nrun 'pip install markdown' to install"
    print(message)
from nbconvert import filters
from pygments import highlight
from IPython.lib.lexers import IPyLexer
from pygments.formatters import HtmlFormatter
from zen_knit.data_types import ChunkOption, OrganizedData, GlobalOption, OrganizedChunk
from zen_knit.formattor.base_formatter import BaseFormatter
from zen_knit.formattor.makrdown_math import MathExtension
from zen_knit.formattor.html_support import htmltemplate, themes
from zen_knit import __version__
import os

class HTMLFormatter(BaseFormatter):
    def __init__(self, organized_data:OrganizedData):
        super().__init__(organized_data)

    @staticmethod
    def _make_css_request(url):
        r = requests.get(url)
        if r.status_code != 200:
            raise Exception(f"CSS URL dont found, status code is {r.status_code}")
        if 200 < r.status_code < 300:
            print(f"css get request return status code {r.status_code}")

        return r.text

    @staticmethod
    def _get_default_theme(css):
        theme_css = themes.get(css)
        return theme_css
        
    def _get_css(self):
        css = self.organized_data.global_options.css
        if 'http' in css:
            return self._make_css_request(css)
        
        results = self._get_default_theme(css)
        
        if results is None:
            file = os.path.abspath(css)
            with open(file, "r") as f:
                results = f.read()

        if results is None:
            raise Exception(f"No CSS found in default themes and file, , currently we are supporting {themes.keys()} ")
            
        return results
    
    def _add_header_fotter(self):
        theme_css = self._get_css()
        self.header = (htmltemplate["header"] %
                {"pygments_css"  : HtmlFormatter().get_style_defs(),
                "theme_css" : theme_css})
        self.footer = htmltemplate["footer"].format(source=self.organized_data.global_options.input_file_name, version=__version__, date=self.organized_data.global_options.date)

    def _parsetitle(self, global_option:GlobalOption):
        self._add_header_fotter()
        title = global_option.title
        author = global_option.author
        date = global_option.date
        lines = ''
        lines += f'\n <H1 class = "title", id="ttitle">{title}</H1> <BR/>'
        if author is not None:
            lines += f'\n <strong>Author:</strong> {author} <BR/>'
        if date is not None:
            lines += f'\n <strong>Date:</strong> {date} <BR/>' 
        
        self.header += lines        

    def _format_docchunk(self, content:OrganizedChunk):
        t = "\n".join(content.str_data.split("\\n"))
        return markdown.markdown(t, extensions=[MathExtension()])
    
    def _format_codechunks(self, content:OrganizedChunk):
        t = filters.ansi2html(content.str_data)
        t = highlight(t, IPyLexer(), HtmlFormatter())

        return t

    def _format_html(self, content: OrganizedChunk):
        if '<table' in content.str_data:
            return "\n".join(content.str_data.split("\\n"))
        else:
            return content.str_data
    
    def _format_image(self, content:OrganizedChunk):
        result = ""
        figstring = ""
        options: ChunkOption
        options = content.complex_data["options"]
        for fig in content.complex_data["plots"]:
            with open(fig, "rb") as f:
                data = f.read()
            fig_base64 = base64.b64encode(data).decode("utf-8")
            figstring += f'<img src="data:image/png;base64,{fig_base64}" width="{options.fig_width}"/>\n' 

        if options.fig_caption:
            if options.name:
                labelstring = f'data-label = "fig:{options.name}"' 
            else:
                labelstring = ""

            result += f"""<figure>
                       {figstring}
                       "<figcaption {labelstring}>{options.fig_caption}</figcaption>\n</figure>"""

        else:
            result += figstring

        return result

    def _build_doc(self):
        self.formatted_doc =  self.header + self.formatted_doc  + self.footer
    
  


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zen_knit',
 'zen_knit.data_types',
 'zen_knit.executor',
 'zen_knit.formattor',
 'zen_knit.formattor.html_support',
 'zen_knit.organizer',
 'zen_knit.parser',
 'zen_knit.reader']

package_data = \
{'': ['*']}

install_requires = \
['Markdown>=3.3.6,<4.0.0',
 'SQLAlchemy>=1.4.29,<2.0.0',
 'click>=8.0.3,<9.0.0',
 'ipykernel>=6.0.0',
 'ipython>=7.0',
 'jupyter-client>=7.1.0,<8.0.0',
 'nbconvert>=6.0.0',
 'nbformat>=5.1.3,<6.0.0',
 'pydantic>=1.8.2,<2.0.0']

entry_points = \
{'console_scripts': ['knit = zen_knit.cli:knit']}

setup_kwargs = {
    'name': 'zen-knit',
    'version': '0.1.7',
    'description': 'Zen-Knit is a formal (PDF), informal (HTML) report generator for data analyst and data scientist who wants to use python. Inspired from Pweave. ',
    'long_description': 'About Zen-Knit:\n---------------\n\nZen-Knit is a formal (PDF), informal (HTML) report generator for data analyst and data scientist who wants to use python. Inspired from Pweave. \nZen-Knit is good for creating reports, tutorials with embedded python. RMarkdown alternative.\n\nFeatures:\n---------\n\n* Python 3.7+ compatibility\n* Support for IPython magics and rich output.\n* **Execute python code** in the chunks and **capture** input and output to a report.\n* **Use hidden code chunks,** i.e. code is executed, but not printed in the output file.\n* Capture matplotlib graphics.\n* Evaluate inline code in documentation chunks marked using ```{ }``` \n* Publish reports from Python scripts. Similar to R markdown.\n* Interactive Plots using plotly\n* integrate it in your process. It will fit your need rather than you need to adjust for tool.\n* custom CSS support (HTTP(s) and local file)\n* direct sql support \n  \nExamples:\n---------\n\n===========\nPDF example\n===========\n\n.. figure:: ./doc/example/screenshots/pdf_code.png\n  :width: 500px\n\n\n.. figure:: ./doc/example/screenshots/pdf_output.png\n  :width: 500px\n\n\n====================\nPDF example with SQL\n====================\n\n.. figure:: ./doc/example/screenshots/pdf_sql_code.png\n  :width: 500px\n\n\n.. figure:: ./doc/example/screenshots/pdf_sql_output.png\n  :width: 500px\n\n============\nHTML example\n============\n.. figure:: ./doc/example/screenshots/html_code.png\n  :width: 500px\n\n.. figure:: ./doc/example/screenshots/html_output.png\n  :width: 500px\n\n.. figure:: ./doc/example/screenshots/html_output_2.png\n  :width: 500px\n\n============================\nHTML example with custom CSS\n============================\n.. figure:: ./doc/example/screenshots/html_cdn_css_code.png\n  :width: 500px\n\n.. figure:: ./doc/example/screenshots/html_custom_css_code.png\n  :width: 500px\n\n=====================\nHTML example with SQL\n=====================\n\n.. figure:: ./doc/example/screenshots/html_sql_code.png\n  :width: 500px\n\n\n.. figure:: ./doc/example/screenshots/html_sql_output.png\n  :width: 500px\n\n\n\nInstall\n-----------------------\n\nFrom PyPi::\n\n  pip install --upgrade zen-knit\n\nor download the source and run::\n\n  python setup.py install\n\n\n\nLicense information\n-------------------\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n"Software"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n\n\nHow to Use it\n-------------\n\n  pip install zen-knit\n\n  knit -f doc/example/html_example.pyz  -ofd doc/example/output/\n  \n  knit -f doc/example/pdf_example.pyz  -ofd doc/example/output/\n\n  python doc/example/demo.py',
    'author': 'Zen',
    'author_email': 'zenreportz@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Zen-Reportz/zen_knit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import io
import os
import base64
from pathlib import Path

from nbconvert import filters
from pygments.formatters.latex import LatexFormatter
from zen_knit import formattor

from zen_knit.data_types import ChunkOption, ExecutedData, OrganizedChunk, OrganizedData
from zen_knit.formattor.html_formatter import HTMLFormatter


mime_extensions = {"image/png" : "png",
                   "image/jpg" : "jpg"}


class BaseOrganizer:
    def __init__(self, executed_data: ExecutedData):
        self.format_started = False
        self.collected_string = ""
        self.fig_folder = None
        self.executed_data = executed_data
        self.formatted_doc = []
        self.organized_data = OrganizedData(
            global_options = self.executed_data.global_options,
            chunks = []
        )
        self._create_output_folder_name()
        self._create_fig_folder()
        self._organize_doc()
        self._create_output_file_name()

    def _create_output_file_name(self):
        global_options = self.organized_data.global_options
        global_options.output_file_name = global_options.input_file_name.split(".")[0] + "."+ global_options.output_format

      
    def _create_output_folder_name(self):
        global_options = self.organized_data.global_options
        
        if global_options.output_file_dir is None:
            global_options.output_file_dir = global_options.input_file_dir
        

    def _create_fig_folder(self):
        output_folder = self.organized_data.global_options.output_file_dir
        Path(output_folder).mkdir(parents=True, exist_ok=True)

        fig_folder = os.path.join(output_folder, self.organized_data.global_options.fig_dir)
        self.fig_folder = fig_folder
        Path(fig_folder).mkdir(parents=True, exist_ok=True)
    

    def _parse_raw(self, data, output_type):
        if data.get("code_text_raw") is not None:
            if self._clean_up(data['code_text_raw']) is not None:
                if output_type in ("code", "sql"):
                    t = {"type": "code", "str_data": data['code_text_raw'] }
                else:
                    t = {"type": "markdown", "str_data": data['code_text_raw'] }
                
                self.organized_data.chunks.append(OrganizedChunk(**t))
            return True
        else:
            return False
    
    def _coder_string(self, data):
        list_ = ["stream", "error"]
        if data["output_type"] is None:
            return False
        
        if data["output_type"] in list_:
            if data["output_type"] == "stream":
                if self._clean_up(data['text']) is not None:
                    t = {"type": "se_data", "str_data": data['text'] }
                    self.organized_data.chunks.append(OrganizedChunk(**t))

            if data["output_type"] == "error":
                t = {"type": "se_data", "str_data": data["evalue"] + filters.strip_ansi("".join(data["traceback"])) }
                self.organized_data.chunks.append(OrganizedChunk(**t))

            return True

        return False

    def _raw_string(self, data):
        if data["output_type"] is None:
            return False

        if data["output_type"] == "execute_result":
            if data.get("data") is not None:
                if 'matplotlib' in data["data"]["text/plain"]:
                    # Doing nothing here
                    return True
                else:
                    if ((data["data"]["text/plain"][0] == "'") or (data["data"]["text/plain"][0] == '"')):
                        temp = data["data"]["text/plain"][1:-1]
                    else:
                        temp = data["data"]["text/plain"]
                    
                    if "<table" in temp:
                        t = {"type": "html_data", "str_data":temp.encode().decode() }
                        self.organized_data.chunks.append(OrganizedChunk(**t))
                        return True
                    # if "BokehJS" in temp:
                    #     t = {"type": "html_data", "str_data": "<script type='text/javascript'>" + temp.encode().decode() + "</script>"  }
                    #     self.organized_data.chunks.append(OrganizedChunk(**t))
                        # return True

                    if self._clean_up(temp) is not None:
                        t = {"type": "e_data", "str_data":temp }
                        self.organized_data.chunks.append(OrganizedChunk(**t))
                        return True
        
            return True           

        return False
    
    def _raw_plots(self, data, chunk_option:ChunkOption):
        if data["output_type"] is None:
            return False

        if data["output_type"] == "display_data":
            plot_infos = self._save_plots(data, chunk_option)
            t = {"type": "plot", "complex_data":{"plots": plot_infos, "options": chunk_option }}
            self.organized_data.chunks.append(OrganizedChunk(**t))
            return True
        return False
    
    def _save_plots(self, data, chunk_option:ChunkOption):
        figs = []
        i = 1
        for m in mime_extensions:
            if m in data["data"]:
                fig_full_path, fig_relative_path = self._build_file(mime_extensions[m], i, chunk_option.fig_caption, chunk_option.name)
                figs.append(fig_relative_path)
                bfig = base64.b64decode(data["data"][m])
                with open(fig_full_path, "wb") as f:
                    f.write(bfig)
                i += 1
        
        return figs
    
    def _build_file(self, extension, index, fig_caption= None, name =None):
        
      
        fig_name = ""
        if fig_caption is not None:
            fig_name = fig_name + "_" +  fig_caption

        if name is not None:
            fig_name = fig_name + "_" + name
        
        fig_name = fig_name + "_" + str(index)
        fig_name = fig_name + "." + extension
        return  os.path.join(self.fig_folder, fig_name), os.path.join(self.fig_folder, fig_name)

    def _interactive_plots(self, data):
        
        if data["output_type"] is None:
            return False

        if data["output_type"] == "display_data":
            if "text/html" in data["data"]:    
                if self.executed_data.global_options.output_format != "html":
                    raise Exception("output format is not HTML")
                else:        
                    t = {"type": "html_data", "str_data":data["data"]["text/html"].encode().decode() }
                    self.organized_data.chunks.append(OrganizedChunk(**t))
                    return True
        return False

    def _organize_doc(self):
        for chunk in self.executed_data.chunks:
            chunk_option = chunk.chunk.options
            results = chunk.results
            for result in results:
                data = result.data
                present = self._parse_raw(data, result.output_type)
                if present:
                    continue
                present = self._coder_string(data)
                if present:
                    continue
               
                present = self._raw_string(data)
                if present:
                    continue

                present = self._interactive_plots(data)
                if present:
                    continue
                
                present = self._raw_plots(data, chunk_option)
                if present:
                    continue
                print("not supported format", data)
            
        
        t = []
        c: OrganizedChunk
        for c in self.organized_data.chunks:
            last_chank: OrganizedChunk
            if len(t)> 0:
                last_chank = t[-1]
            else:
                last_chank = None
            
            if last_chank is None:
                t.append(c)
            else:
                if (c.type == last_chank.type) & (c.type != "plot"):
                    last_chank.str_data = last_chank.str_data + "\n" + c.str_data
                else:
                    t.append(c)
        self.organized_data.chunks = t
    
    @staticmethod
    def _clean_up(doc):
        d = doc.replace(" ", "").replace("\n", "")
        if len(d) != 0:
            return doc
        else:
            return None
        

    #     markdown_file = self.executed_data.global_options.input_file_name.split(".")[0] + ".md"
    #     markdown_file = os.path.join(self.executed_data.global_options.output_file_dir , markdown_file)
    #     with open(markdown_file, "w") as f:
    #         text = "\n".join(self.formatted_doc)
    #         f.write(text)
                


from zen_knit.formattor.html_support import bootstrap
from zen_knit.formattor.html_support import cerulean
from zen_knit.formattor.html_support import skleton
from zen_knit.formattor.html_support import journal
themes = {
    "bootstrap": bootstrap.css,
    "cerulean": cerulean.css,
    "skleton": skleton.css,
    "journal": journal.css

}

htmltemplate = {}

htmltemplate["header"] = \
"""
<!DOCTYPE html>
<HTML>
<HEAD>

<meta charset="UTF-8">
<meta name="viewport" content="width=device-width, initial-scale=1">
<script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/require.js/2.3.6/require.min.js"></script>
<script type="text/x-mathjax-config">
    MathJax.Hub.Config({
    extensions: ["tex2jax.js"],
    jax: ["input/LaTeX", "output/HTML-CSS"],
    "HTML-CSS": { availableFonts: ["TeX"] }
    });
</script>

<script type="text/javascript"
    src="https://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML">
</script>

<style>
%(theme_css)s
</style>

<style>
%(pygments_css)s
</style>





<style>
h1.title {margin-top : 20px}
img {max-width : 100%%}

#From nbconvert
.ansi-black-fg { color: #3E424D; }
.ansi-black-bg { background-color: #3E424D; }
.ansi-black-intense-fg { color: #282C36; }
.ansi-black-intense-bg { background-color: #282C36; }
.ansi-red-fg { color: #E75C58; }
.ansi-red-bg { background-color: #E75C58; }
.ansi-red-intense-fg { color: #B22B31; }
.ansi-red-intense-bg { background-color: #B22B31; }
.ansi-green-fg { color: #00A250; }
.ansi-green-bg { background-color: #00A250; }
.ansi-green-intense-fg { color: #007427; }
.ansi-green-intense-bg { background-color: #007427; }
.ansi-yellow-fg { color: #DDB62B; }
.ansi-yellow-bg { background-color: #DDB62B; }
.ansi-yellow-intense-fg { color: #B27D12; }
.ansi-yellow-intense-bg { background-color: #B27D12; }
.ansi-blue-fg { color: #208FFB; }
.ansi-blue-bg { background-color: #208FFB; }
.ansi-blue-intense-fg { color: #0065CA; }
.ansi-blue-intense-bg { background-color: #0065CA; }
.ansi-magenta-fg { color: #D160C4; }
.ansi-magenta-bg { background-color: #D160C4; }
.ansi-magenta-intense-fg { color: #A03196; }
.ansi-magenta-intense-bg { background-color: #A03196; }
.ansi-cyan-fg { color: #60C6C8; }
.ansi-cyan-bg { background-color: #60C6C8; }
.ansi-cyan-intense-fg { color: #258F8F; }
.ansi-cyan-intense-bg { background-color: #258F8F; }
.ansi-white-fg { color: #C5C1B4; }
.ansi-white-bg { background-color: #C5C1B4; }
.ansi-white-intense-fg { color: #A1A6B2; }
.ansi-white-intense-bg { background-color: #A1A6B2; }

.ansi-bold { font-weight: bold; }
</style>

</HEAD>
<BODY>
    <div class ="container">
        <div class = "row">
            <div class = "col-md-12 twelve columns">
"""



htmltemplate["footer"]="""
    <HR/>
    <div class="footer">
      <p>Published from <a href="{source}">{source}</a>
    using <a href="http://zenreportz.com/zen-knit">Zen Knit</a> {version}
    on {date}.<p></div>

            </div>
        </div>
    </div>
  </BODY>
</HTML>
"""



import io

from nbconvert import filters
from pygments.formatters import LatexFormatter
from subprocess import Popen, PIPE
from pygments import highlight
from IPython.lib.lexers import IPyLexer
from pygments.formatters import LatexFormatter

from zen_knit.data_types import ChunkOption, GlobalOption, OrganizedChunk, OrganizedData
from zen_knit.formattor.base_formatter import BaseFormatter

class TexFormatter(BaseFormatter):
    def __init__(self, organized_data:OrganizedData):
        super().__init__(organized_data)
        self.header = ("""\\documentclass[a4paper,11pt,final]{article}
            \\usepackage{fancyvrb, color, graphicx, hyperref, amsmath, url, textcomp, booktabs}
            \\usepackage{palatino}
            \\usepackage[a4paper,text={16.5cm,25.2cm},centering]{geometry}

            %%Set different options for xetex and luatex
            \\usepackage{iftex}
            \\ifxetex\\usepackage{fontspec}\\fi

            \\ifluatex\\usepackage{fontspec}\\fi

            \\usepackage{xcolor}
            %% ANSI colors from nbconvert
            \\definecolor{ansi-black}{HTML}{3E424D}
            \\definecolor{ansi-black-intense}{HTML}{282C36}
            \\definecolor{ansi-red}{HTML}{E75C58}
            \\definecolor{ansi-red-intense}{HTML}{B22B31}
            \\definecolor{ansi-green}{HTML}{00A250}
            \\definecolor{ansi-green-intense}{HTML}{007427}
            \\definecolor{ansi-yellow}{HTML}{DDB62B}
            \\definecolor{ansi-yellow-intense}{HTML}{B27D12}
            \\definecolor{ansi-blue}{HTML}{208FFB}
            \\definecolor{ansi-blue-intense}{HTML}{0065CA}
            \\definecolor{ansi-magenta}{HTML}{D160C4}
            \\definecolor{ansi-magenta-intense}{HTML}{A03196}
            \\definecolor{ansi-cyan}{HTML}{60C6C8}
            \\definecolor{ansi-cyan-intense}{HTML}{258F8F}
            \\definecolor{ansi-white}{HTML}{C5C1B4}
            \\definecolor{ansi-white-intense}{HTML}{A1A6B2}

            \\hypersetup
            {   pdfauthor = {Pweave},
                pdftitle={Published from %s},
                colorlinks=TRUE,
                linkcolor=black,
                citecolor=blue,
                urlcolor=blue
            }
            \\setlength{\parindent}{0pt}
            \\setlength{\parskip}{1.2ex}
            %% fix for pandoc 1.14
            \\providecommand{\\tightlist}{%%
                \\setlength{\\itemsep}{0pt}\\setlength{\\parskip}{0pt}}
            %s
            """) % (self.organized_data.global_options.input_file_name, LatexFormatter().get_style_defs())
        self.formatted_doc = ''
        self.footer = r"\end{document}"
        self.subheader = "\n\\begin{document}\n"


    def _parsetitle(self, global_option:GlobalOption):
        title = global_option.title
        author = global_option.author
        date = global_option.date
        self.header += f'\n\\title{{{title}}}\n'
        if author is not None:
            self.header += f'\\author{{{author}}}\n'
        if date is not None:
            self.header += f'\\date{{{date}}}\n' 
        self.subheader += "\maketitle\n"
        
    def _format_docchunk(self, content:OrganizedChunk):
        if "tabular" in content.str_data:
            all_data = content.str_data.split("\\n")
            all_data = [t.replace("\n", "").replace("\\\\", "\\")  for t in all_data ]
            all_data = "\n".join(all_data)
        else:
            all_data = content.str_data
            
      
        try:
            pandoc = Popen(["pandoc", "-t", "latex+raw_tex", "-f", "markdown"], stdin=PIPE, stdout=PIPE)
        except:
            raise Exception("ERROR: Can't find pandoc")
        pandoc.stdin.write(all_data.encode('utf-8'))
        transfomed = pandoc.communicate()[0].decode('utf-8')
        return transfomed
    

    def _format_codechunks(self, content:OrganizedChunk):
        transfomed = content.str_data
        transfomed = highlight(transfomed, IPyLexer(), LatexFormatter(verboptions="frame=single,fontsize=\small, xleftmargin=0.5em"))
        transfomed = filters.ansi2latex(transfomed)
        return transfomed

    def _format_image(self, content:OrganizedChunk):

        result = ""
        figstring = ""

        options: ChunkOption   
        options = content.complex_data["options"]
        fig_width =  options.fig_width or "\linewidth"
        fig_caption = options.fig_caption
        fig_env = options.fig_env
        fig_position = options.fig_position
        name =  options.name
        
        if fig_env is not None:
            result += f"\\begin{{{fig_env}}}\n"

        for fig in content.complex_data["plots"]:
            figstring += f"\\includegraphics[width= {fig_width}]{{{fig}}}\n" 

        # Figure environment
        if fig_caption:
            result += f"""\\begin{{figure}}[{fig_position}]
                       \\center
                       {figstring}
                       "\\caption{fig_caption}\n"""
            if name:
                result += f"\label{{fig:{name}}}\n" 
            result += "\\end{figure}\n"

        else:
            result += figstring

        if fig_env is not None:
            result += f"\\end{{{fig_env}}}\n"

        return result
    
    def _build_doc(self):
        self.formatted_doc =  self.header  + self.subheader  + self.formatted_doc + self.footer

    def write_file(self):
        fd = self.organized_data.global_options.output_file_dir
        fn = self.organized_data.global_options.output_file_name.split(".")[0] + ".tex"
        file_  = f"{fd}/{fn}"
        with io.open(file_, 'wt', encoding='utf-8') as f:
            f.write(self.formatted_doc)

        return file_

    def _post_process(self, file_):
        try:
            # pandoc2latex
            # pdflatex
            latex = Popen(["pdflatex", "-output-directory", self.organized_data.global_options.output_file_dir, file_], stdin=PIPE, stdout=PIPE)
            print("Running  pdflatex ...")
        except Exception as e:
            print(e)
            print("Can't find pdflatex no pdf produced!")
            return
        x = latex.communicate()[0].decode('utf-8')
        print("\n".join(x.splitlines()[-2:]))